// AkosBrowser.cpp
// By Nolan Check

#include "AkosBrowser.h"

bool AkosBrowser::Initialize(LPTSTR lpGamePath, LPTSTR lpGameName)
{
	Destroy();

	m_lpGamePath = new TCHAR[strlen(lpGamePath)];
	if (!m_lpGamePath)
	{
		ErrorMessage("Out of memory.");
		return false;
	}
	m_lpGameName = new TCHAR[strlen(lpGameName)];
	if (!m_lpGameName)
	{
		ErrorMessage("Out of memory.");
		return false;
	}

	strcpy(m_lpGamePath, lpGamePath);
	strcpy(m_lpGameName, lpGameName);

	m_pScummResourceManager = new ScummResourceManager(m_lpGamePath, m_lpGameName);
	if (!m_pScummResourceManager)
	{
		ErrorMessage("Out of memory.");
		return false;
	}

	if (!m_pScummResourceManager->Initialize())
	{
		return false;
	}

	return true;
}

void AkosBrowser::Destroy()
{
	if (m_pAkos)
	{
		delete m_pAkos;
		m_pAkos = NULL;
	}
	if (m_pAkosData)
	{
		free(m_pAkosData);
		m_pAkosData = NULL;
	}
	if (m_pScummResourceManager)
	{
		m_pScummResourceManager->Destroy();
		delete m_pScummResourceManager;
		m_pScummResourceManager = NULL;
	}
	if (m_lpGameName)
	{
		delete[] m_lpGameName;
		m_lpGameName = NULL;
	}
	if (m_lpGamePath)
	{
		delete[] m_lpGamePath;
		m_lpGamePath = NULL;
	}
	m_fCostumeLoaded = false;
	m_fFrameLoaded = false;
}

void AkosBrowser::LoadCostume(DWORD dwCostume)
{
	if (dwCostume > m_pScummResourceManager->GetNumCostumes())
	{
		ErrorMessage("Costume out of range.");
		return;
	}

	if (m_pAkos)
	{
		delete m_pAkos;
		m_pAkos = NULL;
	}
	if (m_pAkosData)
	{
		free(m_pAkosData);
		m_pAkosData = NULL;
	}

	m_dwCurrentCostume = dwCostume;
	m_fCostumeLoaded = false;
	switch (m_pScummResourceManager->LoadCostume(m_pAkosData, m_dwCurrentCostume))
	{
	case LI_DISK_NOT_PRESENT:
		{
			ClearDisplay();
			char c[256];
			wsprintf(c, "Costume %d is not present on this disk.", m_dwCurrentCostume);
			DisplayMessage(c);
		}
		break;
	case LI_SUCCESS:
		m_pAkos = new Akos(m_pAkosData);
		if (!m_pAkos)
		{
			ErrorMessage("Out of memory.");
			return;
		}
		if (!m_pAkos->Initialize())
		{
			return;
		}
		m_fCostumeLoaded = true;
		LoadFrame(1);
		break;
	}
}

void AkosBrowser::AdvanceCostume()
{
	if (m_dwCurrentCostume + 1 > m_pScummResourceManager->GetNumCostumes())
	{
		return;
	}

	LoadCostume(m_dwCurrentCostume + 1);
}

void AkosBrowser::BackCostume()
{
	if (m_dwCurrentCostume - 1 <= 0)
	{
		return;
	}

	LoadCostume(m_dwCurrentCostume - 1);
}

void AkosBrowser::LoadFrame(WORD wFrame)
{
	if (!m_fCostumeLoaded)
	{
		return;
	}

	if (wFrame > m_pAkos->GetMaxFrames())
	{
		ErrorMessage("Frame out of range.");
		return;
	}

	m_fFrameLoaded = false;
	if (m_pAkos->LoadFrame(wFrame))
	{
		m_fFrameLoaded = true;
	}
}

void AkosBrowser::AdvanceFrame()
{
	if (!m_fCostumeLoaded)
	{
		return;
	}

	if (m_pAkos->GetCurrentFrame() + 1 > m_pAkos->GetMaxFrames())
	{
		return;
	}

	LoadFrame(m_pAkos->GetCurrentFrame() + 1);
}

void AkosBrowser::BackFrame()
{
	if (!m_fCostumeLoaded)
	{
		return;
	}

	if (m_pAkos->GetCurrentFrame() - 1 <= 0)
	{
		return;
	}

	LoadFrame(m_pAkos->GetCurrentFrame() - 1);
}

void AkosBrowser::DrawFrame(Display& display)
{
	if (!m_fCostumeLoaded)
	{
		return;
	}
	if (!m_fFrameLoaded)
	{
		return;
	}

	Sprite sprite;
	if (m_pAkos->DrawFrame(sprite, 0, 0))
	{
		if (m_fZoom)
		{
			Sprite* pZoomed = new Sprite;
			if (!pZoomed)
			{
				ErrorMessage("Out of memory.");
				return;
			}

			if (!pZoomed->Initialize(sprite.GetX() * 2, sprite.GetY() * 2, sprite.GetWidth() * 2, sprite.GetHeight() * 2, sprite.GetNumColors()))
			{
				return;
			}

			memcpy(pZoomed->GetPalette(), sprite.GetPalette(), sprite.GetNumColors() * 3);

			BYTE* pSrc = sprite.GetBits();
			BYTE* pDst = pZoomed->GetBits();
			for (WORD wh = 0; wh < sprite.GetHeight(); wh++)
			{
				for (WORD ww = 0; ww < sprite.GetWidth(); ww++)
				{
					*pDst = *pSrc;
					*(pDst + 1) = *pSrc;
					pDst += 2;
					pSrc++;
				}
				memcpy(pDst, pDst - pZoomed->GetWidth(), pZoomed->GetWidth());
				pDst += pZoomed->GetWidth();
			}

			display.Clear();
			display.DrawSprite(*pZoomed);
			pZoomed->Destroy();
		}
		else
		{
			display.Clear();
			display.DrawSprite(sprite);
		}
		sprite.Destroy();

		char c[256];
		wsprintf(c, "Costume %d, Frame %d / %d", m_dwCurrentCostume, m_pAkos->GetCurrentFrame(), m_pAkos->GetMaxFrames());
		DisplayMessage(c);
	}
	else
	{
		display.Clear();
		DisplayMessage("Failed to draw frame.");
	}
}