// AkosView.cpp
// By Nolan Check

#include "AkosView.h"
#include "Display.h"
#include "AkosBrowser.h"
#include "resource.h"
#include <afxres.h>
#include <commdlg.h>

#define WINDOW_NAME   "AkosView"
#define WINDOW_CLASS  "AKOSVIEW"
#define WINDOW_WIDTH  640
#define WINDOW_HEIGHT 480

HINSTANCE g_hInstance;
HWND      g_hWindow;

Display      g_display;
AkosBrowser* g_pAkosBrowser = NULL;

void ClearDisplay()
{
	g_display.Clear();
}

void ErrorMessage(LPCTSTR lpMessage)
{
	MessageBox(g_hWindow, lpMessage, "Error", MB_ICONERROR | MB_OK);
}

void DisplayMessage(LPCTSTR lpMessage)
{
	RECT rc;
	GetClientRect(g_hWindow, &rc);
	DrawText(g_display.GetHDC(), lpMessage, (int)strlen(lpMessage), &rc, DT_CENTER);
	InvalidateRect(g_hWindow, &rc, FALSE);
}

void PleaseOpenFilePrompt()
{
	g_display.Clear();
	DisplayMessage("Please open a file.");
}

bool OpenScumm(LPTSTR lpScummPath, LPTSTR lpScummGameName);
void CloseScumm();
bool OpenFileDialog()
{
	TCHAR cFilePath[MAX_PATH];
	TCHAR cFileName[MAX_PATH];
	ZeroMemory(cFilePath, sizeof(TCHAR) * MAX_PATH);
	ZeroMemory(cFileName, sizeof(TCHAR) * MAX_PATH);

	OPENFILENAME ofn;
	ofn.lStructSize       = sizeof(OPENFILENAME);
	ofn.hwndOwner         = g_hWindow;
	ofn.hInstance         = NULL;
	ofn.lpstrFilter       = "SCUMM Index Files\0*.LA0;*.HE0;*.000\0";
	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxCustFilter    = 0;
	ofn.nFilterIndex      = 1;
	ofn.lpstrFile         = cFilePath;
	ofn.nMaxFile          = MAX_PATH;
	ofn.lpstrFileTitle    = cFileName;
	ofn.nMaxFileTitle     = MAX_PATH;
	ofn.lpstrInitialDir   = NULL;
	ofn.lpstrTitle        = NULL;
	ofn.Flags             = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
	ofn.nFileOffset       = 0;
	ofn.nFileExtension    = 0;
	ofn.lpstrDefExt       = NULL;
	ofn.lCustData         = NULL;
	ofn.lpfnHook          = NULL;
	ofn.lpTemplateName    = NULL;
	if (!GetOpenFileName(&ofn))
	{
		return false;
	}

	cFilePath[ofn.nFileOffset] = '\0';
	cFileName[ofn.nFileExtension - strlen(cFilePath) - 1] = '\0';

	if (!OpenScumm(cFilePath, cFileName))
	{
		CloseScumm();
		PleaseOpenFilePrompt();
		return false;
	}

	return true;
}

bool OpenScumm(LPTSTR lpScummPath, LPTSTR lpScummGameName)
{
	CloseScumm();

	g_pAkosBrowser = new AkosBrowser;
	if (!g_pAkosBrowser)
	{
		ErrorMessage("Out of memory.");
		return false;
	}

	if (!g_pAkosBrowser->Initialize(lpScummPath, lpScummGameName))
	{
		return false;
	}
	
	g_pAkosBrowser->LoadCostume(1);
	g_pAkosBrowser->DrawFrame(g_display);

	return true;
}

void CloseScumm()
{
	if (g_pAkosBrowser)
	{
		g_pAkosBrowser->Destroy();
		g_pAkosBrowser = NULL;
	}
}

void DestroyAkosView()
{
	CloseScumm();
	g_display.Destroy();
}

LRESULT CALLBACK WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	switch (uMsg)
	{
	case WM_PAINT:
		PAINTSTRUCT ps;
		hdc = BeginPaint(hwnd, &ps);
		BitBlt(hdc, ps.rcPaint.left, ps.rcPaint.top, ps.rcPaint.right - ps.rcPaint.left, ps.rcPaint.bottom - ps.rcPaint.top, g_display.GetHDC(), ps.rcPaint.left, ps.rcPaint.top, SRCCOPY);
		EndPaint(hwnd, &ps);
		return 0;
	case WM_KEYDOWN:
		switch (wParam)
		{
		case VK_RIGHT:
			if (g_pAkosBrowser)
			{
				g_pAkosBrowser->AdvanceFrame();
				g_pAkosBrowser->DrawFrame(g_display);
			}
			return 0;
		case VK_LEFT:
			if (g_pAkosBrowser)
			{
				g_pAkosBrowser->BackFrame();
				g_pAkosBrowser->DrawFrame(g_display);
			}
			return 0;
		case VK_UP:
			if (g_pAkosBrowser)
			{
				g_pAkosBrowser->AdvanceCostume();
				g_pAkosBrowser->DrawFrame(g_display);
			}
			return 0;
		case VK_DOWN:
			if (g_pAkosBrowser)
			{
				g_pAkosBrowser->BackCostume();
				g_pAkosBrowser->DrawFrame(g_display);
			}
			return 0;
		case VK_ADD:
			if (g_pAkosBrowser)
			{
				g_pAkosBrowser->SetZoom(true);
				g_pAkosBrowser->DrawFrame(g_display);
			}
			return 0;
		case VK_SUBTRACT:
			if (g_pAkosBrowser)
			{
				g_pAkosBrowser->SetZoom(false);
				g_pAkosBrowser->DrawFrame(g_display);
			}
			return 0;
		}
		break;
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case ID_FILE_OPEN:
			OpenFileDialog();
			return 0;
		case IDCLOSE:
			DestroyWindow(hwnd);
			return 0;
		}
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;
	}

	return DefWindowProc(hwnd, uMsg, wParam, lParam);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	g_hInstance = hInstance;

	WNDCLASS wc = {
		0,
		WindowProc,
		0,
		0,
		g_hInstance,
		LoadIcon(g_hInstance, MAKEINTRESOURCE(IDI_ICON)),
		LoadCursor(NULL, IDC_ARROW),
		NULL,
		MAKEINTRESOURCE(IDR_MENU),
		WINDOW_CLASS
	};
	RegisterClass(&wc);

	RECT rcWindow = {
		0, 0, WINDOW_WIDTH, WINDOW_HEIGHT
	};
	AdjustWindowRect(&rcWindow, WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX, TRUE);
	g_hWindow = CreateWindow(WINDOW_CLASS, WINDOW_NAME, WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX, CW_USEDEFAULT, CW_USEDEFAULT, rcWindow.right - rcWindow.left, rcWindow.bottom - rcWindow.top, NULL, NULL, g_hInstance, NULL);
	if (!g_hWindow)
	{
		DestroyAkosView();
		return 1;
	}

	if (!g_display.Initialize(g_hWindow))
	{
		DestroyAkosView();
		return 1;
	}

	PleaseOpenFilePrompt();

	ShowWindow(g_hWindow, nShowCmd);

	MSG msg;
	while (GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	DestroyAkosView();
	return (int)msg.wParam;
}