// Scumm.h
// By Nolan Check

#pragma once

#include "AkosView.h"
#include "File.h"

enum LIRETURNCODE
{
	LI_FATAL_ERROR,
	LI_DISK_NOT_PRESENT,
	LI_SUCCESS
};

#include <pshpack1.h>
struct BLOCKHEADER
{
	DWORD dwName;
	DWORD dwSize;
};
#include <poppack.h>

class ScummDisk
{
public:
	ScummDisk()
	{
		m_pRoomOffsets = NULL;
	};

	bool Initialize(File* pFile);
	void Destroy();
	bool SeekRoom(File* pFile, BYTE bRoom);
private:
	bool ParseLOFF(File* pFile);

	BYTE   m_numRooms;
	DWORD* m_pRoomOffsets;
};

class ScummDirectory
{
public:
	ScummDirectory()
	{
		m_dwNumItems = 0;
		m_pRooms     = NULL;
		m_pOffsets   = NULL;
	};

	bool Parse(bool fScummV8, BYTE* pData);
	void Destroy();

	DWORD GetNumItems()
	{
		return m_dwNumItems;
	};
	BYTE GetItemRoom(DWORD dwItem)
	{
		return m_pRooms[dwItem];
	};
	DWORD GetItemOffset(DWORD dwItem)
	{
		return m_pOffsets[dwItem];
	};
private:
	DWORD  m_dwNumItems;
	BYTE*  m_pRooms;
	DWORD* m_pOffsets;
};

class ScummResourceManager
{
public:
	ScummResourceManager(LPCTSTR lpScummPath, LPCTSTR lpScummGameName)
	{
		for (int i = 0; i < 10; i++)
		{
			m_pDisks[i] = NULL;
		}
		for (int i = 0; i < 10; i++)
		{
			m_pScummDisks[i] = NULL;
		}
		m_lpScummPath = lpScummPath;
		m_lpScummGameName = lpScummGameName;
	};

	bool         Initialize();
	void         Destroy();
	LIRETURNCODE LoadCostume(BYTE*& pMem, DWORD dwCostume);

	DWORD GetNumCostumes()
	{
		return m_dirCostumes.GetNumItems();
	};
private:
	bool DetectDisks();
	bool ParseIndexDisk();

	File*      m_pDisks[10];
	ScummDisk* m_pScummDisks[10];
	LPCTSTR    m_lpScummPath;
	LPCTSTR    m_lpScummGameName;

	ScummDirectory m_dirRooms;
	ScummDirectory m_dirCostumes;
};